import urllib.request
import yaml

from karamel.exception import PackagesFileNotFound

def read_packages_file(packages_file_url):
    try:
        response = urllib.request.urlopen(packages_file_url)
        content = response.read()
    except ValueError:
        try:
            stream = open(packages_file_url, 'r')
            content = stream.read()
            stream.close()
        except:
            raise PackagesFileNotFound(packages_file_url)

    return yaml.safe_load(content)



import os, re, git

from karamel.packages import read_packages_file
from karamel.exception import KaramelException
from karamel.logger import logger

from .command import Command

__command__ = 'update'
__description__ = 'Update a package'

class UpdateCommand(Command):

    def __init__(self, packages_file_url, package_install_dir):
        super().__init__(__command__, __description__)
        self.add_argument('package')
        self.packages_file_url = packages_file_url
        self.package_install_dir = os.path.expanduser(package_install_dir)

    def callback(self, args):
        packages = read_packages_file(self.packages_file_url)

        if self.package_install_dir != '' and not os.path.isdir(self.package_install_dir):
            os.makedirs(self.package_install_dir)

        if args.package in packages:
            package = packages[args.package]
            package_path = os.path.join(self.package_install_dir, args.package)

            if not os.path.isdir(package_path):
                raise KaramelException('Package is not installed')
            else:
                logger.info('Updating {}'.format(args.package))
                package_path = os.path.join(self.package_install_dir, args.package)
                git.Git(package_path).pull()
                logger.info('Successfully updated {}'.format(args.package))

        else:
            raise KaramelException('Package not found')



import argparse

class Command:

    def __init__(self, command_name, description):
        self.parser = argparse.ArgumentParser(prog=command_name, description=description, add_help=False)
        self.parser.set_defaults(callback=self)
        self.subparsers = None

    def run(self):
        args = self.parser.parse_args()
        args.callback(args)

    def add_command(self, command, description='', help=''):
        if self.subparsers == None:
            self.subparsers = self.parser.add_subparsers(help=help)
        subparser = self.subparsers.add_parser(command.parser.prog, help=command.parser.description)
        subparser.__dict__ = command.parser.__dict__;

    def add_argument(self, *args, **kwargs):
        self.parser.add_argument(*args, **kwargs)

    def callback(self, args):
        pass

    def __call__(self, args):
        self.callback(args)



import re

from karamel.packages import read_packages_file
from karamel.logger import logger

from .command import Command

__command__ = 'search'
__description__ = 'Search package'

class SearchCommand(Command):

    def __init__(self, packages_file_url):
        super().__init__(__command__, __description__)
        self.add_argument('pattern')
        self.packages_file_url = packages_file_url

    def callback(self, args):
        packages = read_packages_file(self.packages_file_url)
        packages_name = list(packages.keys())
        r = re.compile('.*{}.*'.format(args.pattern))
        packages_matching_pattern = filter(r.match, packages_name)
        
        for name in packages_matching_pattern:
            package = packages[name]
            logger.info('{}'.format(name))
            logger.info('  {}'.format(package['description']))
            logger.info('  {}'.format(package['url']))
            logger.info('')



import os, re, git

from karamel.packages import read_packages_file
from karamel.exception import KaramelException
from karamel.logger import logger

from .command import Command

__command__ = 'install'
__description__ = 'Install a package'

class InstallCommand(Command):

    def __init__(self, packages_file_url, package_install_dir):
        super().__init__(__command__, __description__)
        self.add_argument('package')
        self.packages_file_url = packages_file_url
        self.package_install_dir = os.path.expanduser(package_install_dir)

    def callback(self, args):
        packages = read_packages_file(self.packages_file_url)

        if self.package_install_dir != '' and not os.path.isdir(self.package_install_dir):
            os.makedirs(self.package_install_dir)

        if args.package in packages:
            package = packages[args.package]
            package_path = os.path.join(self.package_install_dir, args.package)

            if not os.path.isdir(package_path):
                logger.info('Downloading {}'.format(args.package))
                git.Git(self.package_install_dir).clone(package['url'])
                logger.info('Installing {}'.format(args.package))
                logger.info('Successfully installed {}'.format(args.package))
            else:
                raise KaramelException('Package already installed: {} in {}'.format(args.package, package_path))

        else:
            raise KaramelException('Package not found')



import logging

logger = logging.getLogger('karamel')
logger.setLevel(logging.INFO)
stream_handler = logging.StreamHandler()
formatter = logging.Formatter('%(message)s')
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.INFO)
logger.addHandler(stream_handler)



#!/usr/bin/env python

from setuptools import setup, find_packages

__version__ = '0.1.4'
__program__ = 'karamel'

setup(
    name=__program__,
    version=__version__,
    description='Karamel is a library let you build powerful Package Manager based on Git.',
    author='Noel Martignoni',
    author_email='noel@martignoni.fr',
    url='https://gitlab.com/{0}/{0}'.format(__program__),
    scripts=['scripts/{}'.format(__program__)],
    install_requires=['future', 'pyyaml', 'GitPython'],
    packages=find_packages(exclude=['tests*']),
)



import os, re, git, shutil

from karamel.packages import read_packages_file
from karamel.exception import KaramelException
from karamel.logger import logger

from .command import Command

__command__ = 'uninstall'
__description__ = 'Uninstall a package'

class UninstallCommand(Command):

    def __init__(self, packages_file_url, package_install_dir):
        super().__init__(__command__, __description__)
        self.add_argument('package')
        self.add_argument('-y', '--yes', action='store_true')
        self.packages_file_url = packages_file_url
        self.package_install_dir = os.path.expanduser(package_install_dir)

    def callback(self, args):
        packages = read_packages_file(self.packages_file_url)

        if self.package_install_dir != '' and not os.path.isdir(self.package_install_dir):
            os.makedirs(self.package_install_dir)

        packages_installed = os.listdir(self.package_install_dir)

        if args.package in packages_installed:
            package_path = os.path.join(self.package_install_dir, args.package)
            logger.info('Uninstalling {}:'.format(args.package))
            logger.info('  {}'.format(package_path))

            install = False
            if not args.yes:
                response = input('Proceed (y/n)? ')
                if response == 'y':
                    install = True
                elif response != 'n':
                    raise KaramelException('Bad response')
            else:
                install = True

            if install:
                shutil.rmtree(package_path)
                logger.info('Successfully uninstalled {}'.format(args.package))

        else:
            raise KaramelException('Package not installed: cannot find {} in {}'.format(args.package, self.package_install_dir))



class KaramelException(Exception):
    pass

packages_file_not_found_message = \
'''
Cannot find the file {}'.format(packages_file_url).
Check if the url or the path is valid and the file exist.
If the package file is online also check your connection to Internet.
'''

class PackagesFileNotFound(KaramelException):
    def __init__(self, packages_file):
        message = packages_file_not_found_message.format(packages_file)
        super().__init__(message)
